#ifndef COMPILER_HEADER
#define COMPILER_HEADER

#include "TypeDefs.h"
#include "Absyn.h"
#include "Skeleton.h"
#include <filesystem>
#include <utility>
#include "codeGen/Variable.h"
#include "ParseError.h"
#include "Info.h"
#include "codeGen/Quadruple.h"
#include "codeGen/BasicBlock.h"
#include "codeGen/QuadrupleGenerator.h"

using Op = Quadruple::Op;
class Compiler {
public:
	Compiler(const std::string& f, shared_ptr<Scope> s) : file(f), buf(), scope(s), quadGen(std::move(s)) {};

	static const std::string extension;
	static void externalCommand(std::filesystem::path file);

	string compile(Visitable *v);

	static std::string getVirtName(const ClassInfoPtr& c) {
		return c->name + ":virt_table";
	}
private:
	std::filesystem::path file;
	std::stringstream buf;
	shared_ptr<Scope> scope;
	QuadrupleGenerator quadGen;

	void compileFunction(const FunctionInfoPtr& f);

	static std::string mangleFunctionName(const FunctionInfoPtr& f) {
		if (auto c = f->klass.lock()) {
			return c->name + "::" + f->name;
		}
		return f->name;
	}
};

#endif
